package com.hfkj.openapi.service;

import com.alibaba.fastjson.JSONObject;
import com.hfkj.channel.gas.newlink.NewLinkRequestService;
import com.hfkj.channel.gas.shell.CqShellPetroleumRequestService;
import com.hfkj.common.exception.BaseException;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.pay.util.SignatureUtil;
import com.hfkj.common.utils.DateUtil;
import com.hfkj.common.utils.HttpsUtils;
import com.hfkj.common.utils.RandomUtils;
import com.hfkj.entity.*;
import com.hfkj.model.GasModel;
import com.hfkj.model.GasOilPriceModel;
import com.hfkj.openapi.model.request.RequestOrderDetailModel;
import com.hfkj.openapi.model.request.RequestOrderPushModel;
import com.hfkj.openapi.model.request.RequestOrderRefundModel;
import com.hfkj.openapi.model.response.OrderRefundNotifyModel;
import com.hfkj.openapi.model.response.ResponseOrderDetailModel;
import com.hfkj.openapi.model.response.ResponseOrderPushModel;
import com.hfkj.service.BsDeviceService;
import com.hfkj.service.agent.BsAgentApiAccountService;
import com.hfkj.service.agent.BsAgentApiParamService;
import com.hfkj.service.agent.BsAgentMerService;
import com.hfkj.service.agent.BsAgentService;
import com.hfkj.service.gas.BsGasClassGroupTaskService;
import com.hfkj.service.gas.BsGasOrderService;
import com.hfkj.service.gas.BsGasService;
import com.hfkj.service.merchant.BsMerchantAccountService;
import com.hfkj.service.merchant.BsMerchantChainBrandAccountService;
import com.hfkj.service.merchant.BsMerchantService;
import com.hfkj.service.order.BsOrderRefundService;
import com.hfkj.service.order.BsOrderService;
import com.hfkj.service.sec.SecDictionaryService;
import com.hfkj.sysenum.gas.GasOrderCreateType;
import com.hfkj.sysenum.gas.GasOrderReceiptStatusEnum;
import com.hfkj.sysenum.gas.OrderOilStatus;
import com.hfkj.sysenum.merchant.MerchantAccountRecordSourceTypeEnum;
import com.hfkj.sysenum.merchant.MerchantChainBrandSettleTypeEnum;
import com.hfkj.sysenum.merchant.MerchantSourceTypeEnum;
import com.hfkj.sysenum.merchant.MerchantStatusEnum;
import com.hfkj.sysenum.order.OrderPayTypeEnum;
import com.hfkj.sysenum.order.OrderRefundStatusEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

/**
 * @className: ApiGasOrderService
 * @author: HuRui
 * @date: 2024/11/12
 **/
@Component("apiGasOrderService")
public class ApiGasOrderService {
    @Resource
    private BsAgentMerService agentMerService;
    @Resource
    private BsAgentService agentService;
    @Resource
    private BsMerchantService merchantService;
    @Resource
    private BsMerchantAccountService merchantAccountService;
    @Resource
    private BsMerchantChainBrandAccountService merchantChainBrandAccountService;
    @Resource
    private BsGasService gasService;
    @Resource
    private BsOrderService orderService;
    @Resource
    private BsGasOrderService gasOrderService;
    @Resource
    private BsGasClassGroupTaskService gasClassGroupTaskService;
    @Resource
    private BsDeviceService deviceService;
    @Resource
    private SecDictionaryService dictionaryService;
    @Resource
    private BsAgentApiAccountService agentApiAccountService;
    @Resource
    private BsAgentApiParamService agentApiParamService;

    @Resource
    private BsOrderRefundService orderRefundService;
    /**
     * 推送加油订单
     * @param request
     * @return
     */
    @Transactional(propagation= Propagation.REQUIRES_NEW,rollbackFor= {RuntimeException.class})
    public ResponseOrderPushModel pushOrder(Long agentId,RequestOrderPushModel request) throws Exception {
        // 交易油站权限
        BsAgentMer agentMer = agentMerService.getDetailByAgent(agentId, request.getGasNo());
        if (agentMer == null) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "油站不可用");
        }
        // 查询油站
        GasModel gasDetail = gasService.getGasDetail(request.getGasNo());
        if (gasDetail == null) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "未找到油站信息");
        }
        if (!gasDetail.getStatus().equals(MerchantStatusEnum.status1.getNumber())) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "油站不可用");
        }
        // 校验油品
        List<GasOilPriceModel> gasOilPriceModels = gasDetail.getGasOilPrice().stream().filter(o -> o.getOilNo().equals(request.getOilNo())).collect(Collectors.toList());
        if (gasOilPriceModels.isEmpty()) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "油品不可用");
        }
        // 油价
        GasOilPriceModel oilPrice = gasOilPriceModels.get(0);
        // 校验枪号
        if (oilPrice.getGasOilGunNo().stream().noneMatch(o -> o.getGunNo().equals(request.getGunNo()))) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "枪号不可用");
        }
        // 交易订单号
        if (gasOrderService.getDetailByOrderNo(request.getOrderNo()) != null) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "订单号已存在");
        }
        // 查询代理商
        BsAgent agent = agentService.getAgentById(agentId);
        if (agent == null) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "未知的接入方");
        }
        // 创建加油订单
        BsGasOrder gasOrder = new BsGasOrder();
        gasOrder.setAgentId(agent.getId());
        gasOrder.setAgentName(agent.getName());
        gasOrder.setAbnormal(false);
        gasOrder.setMerNo(gasDetail.getMerNo());
        gasOrder.setChannelType(gasDetail.getSourceType());
        gasOrder.setOrderNo(request.getOrderNo());
        gasOrder.setUserPhone(request.getUserPhone());
        gasOrder.setMerChainBrandId(gasDetail.getChainBrandId());
        gasOrder.setMerChainBrandName(gasDetail.getChainBrandName());
        gasOrder.setMerId(gasDetail.getId());
        gasOrder.setMerName(gasDetail.getMerName());
        gasOrder.setMerAddress(gasDetail.getAddress());
        gasOrder.setStatus(OrderOilStatus.STATUS2.getNumber());
        gasOrder.setReceiptStatus(GasOrderReceiptStatusEnum.status0.getStatus());

        // 结算信息
        gasOrder.setTotalDeductionPrice(new BigDecimal("0"));
        gasOrder.setDeductionCouponPrice(new BigDecimal("0"));
        gasOrder.setDeductionOilPrice(new BigDecimal("0"));
        gasOrder.setPayablePrice(request.getRefuelingAmount());
        gasOrder.setPayablePrice(request.getRefuelingAmount());
        gasOrder.setGasRefuelPrice(request.getRefuelingAmount());
        gasOrder.setGasSettlePrice(gasOrder.getGasRefuelPrice());
        gasOrder.setGasAgentSettlePrice(gasOrder.getGasRefuelPrice());

        // 油价信息
        gasOrder.setGasRefuelPrice(request.getRefuelingAmount());
        gasOrder.setGasOilNo(request.getOilNo());
        gasOrder.setGasGunNo(request.getGunNo());
        gasOrder.setGasOilType(oilPrice.getOilType());
        gasOrder.setGasPriceGun(oilPrice.getPriceGun());
        gasOrder.setGasPriceVip(oilPrice.getPriceGun());
        gasOrder.setGasPriceCost(oilPrice.getPriceGun());
        gasOrder.setGasPriceOfficial(oilPrice.getPriceOfficial());
        gasOrder.setGasPricePlatform(oilPrice.getPriceGun());
        gasOrder.setGasOilLiters(gasOrder.getGasSettlePrice().divide(gasOrder.getGasPriceGun(), 2, BigDecimal.ROUND_HALF_UP));
        gasOrder.setGasDiscount(new BigDecimal("0"));
        gasOrder.setGasOilSubsidy(new BigDecimal("0"));
        gasOrder.setGasLitersPreferences(new BigDecimal("0"));
        gasOrder.setGasPricePreferences(new BigDecimal("0"));
        gasOrder.setCreateType(GasOrderCreateType.TYPE2.getNumber());

        // 油站是否开启班组
        BsGasClassGroupTask groupTask = gasClassGroupTaskService.getCurrentTaskByMerId(gasDetail.getId());
        if (groupTask != null) {
            gasOrder.setGasClassGroupId(groupTask.getGasClassGroupId());
            gasOrder.setGasClassGroupName(groupTask.getGasClassGroupName());
            gasOrder.setGasClassGroupTaskId(groupTask.getId());
        }
        gasOrderService.addGasOrder(gasOrder);

        // 代理商扣款
        Map<String,Object> consumeParam = new HashMap<>();
        consumeParam.put("sourceType", MerchantAccountRecordSourceTypeEnum.type2.getType());
        consumeParam.put("sourceOrderNo", gasOrder.getOrderNo());
        consumeParam.put("sourceOrderContent", "加油订单："+gasOrder.getOrderNo());
        agentApiAccountService.consume(agentId, gasOrder.getGasAgentSettlePrice(), consumeParam);

        // 处理支付后的加油业务
        oilHandle(gasOrder);
        // 加油业务是否异常
        if (gasOrder.getAbnormal().equals(true)) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, gasOrder.getAbnormalContent());
        }
        ResponseOrderPushModel response = new ResponseOrderPushModel();
        response.setOrderNo(gasOrder.getOrderNo());
        response.setChannelType(gasOrder.getChannelType());
        response.setChannelOrderNo(gasOrder.getChannelOrderNo());
        return response;
    }

    /**
     * 退款订单
     * @param request
     * @return
     * @throws Exception
     */
    @Transactional(propagation= Propagation.REQUIRES_NEW,rollbackFor= {RuntimeException.class})
    public String refundOrder(RequestOrderRefundModel request) {
        // 查询加油订单
        BsGasOrder gasOrder = gasOrderService.getDetailByOrderNo(request.getOrderNo());
        if (gasOrder == null) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "未找到订单号");
        }
        if (!gasOrder.getStatus().equals(OrderOilStatus.STATUS2.getNumber())
            || gasOrder.getStatus().equals(OrderOilStatus.STATUS6.getNumber())) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "订单无法退款");
        }
        // 创建退款订单
        BsOrderRefund orderRefund = new BsOrderRefund();
        orderRefund.setMerId(gasOrder.getMerId());
        orderRefund.setMerNo(gasOrder.getMerNo());
        orderRefund.setOrderNo(gasOrder.getOrderNo());
        orderRefund.setUserPhone(gasOrder.getUserPhone());
        // 退款订单号 退款标识：R + 时间 + 随机3位数字
        orderRefund.setRefundOrderNo("R"+ DateUtil.date2String(new Date(), "yyMMddHHmmss") + RandomUtils.number(3, false));
        orderRefund.setRefundPayType(gasOrder.getPayType());
        // 全部退款
        orderRefund.setRefundPrice(gasOrder.getGasRefuelPrice());
        orderRefund.setRefundIntegral(0L);
        orderRefund.setRefundRemark(request.getRefundReason());
        orderRefund.setRefundStatus(OrderRefundStatusEnum.status1.getCode());
        orderRefund.setCreateType(GasOrderCreateType.TYPE2.getNumber());
        orderRefundService.editData(orderRefund);

        ResponseOrderPushModel response = new ResponseOrderPushModel();
        response.setOrderNo(gasOrder.getOrderNo());
        response.setChannelType(gasOrder.getChannelType());
        response.setChannelOrderNo(gasOrder.getChannelOrderNo());

        // 创建一个单线程的线程池
        ExecutorService singleThreadExecutor = Executors.newSingleThreadExecutor();
        // 异步记录登录信息
        singleThreadExecutor.submit(new Runnable() {
            @Override
            public void run() {
                if (!gasOrder.getChannelType().equals(MerchantSourceTypeEnum.type2.getNumber())) {
                    try {
                        // 退款
                        agentApiAccountService.refund(gasOrder.getAgentId(),gasOrder.getGasRefuelPrice(), orderRefund);
                        orderRefund.setRefundStatus(OrderRefundStatusEnum.status2.getCode());
                        orderRefund.setFinishTime(new Date());
                        orderRefundService.editData(orderRefund);
                    } catch (Exception e) {
                        orderRefund.setRefundStatus(OrderRefundStatusEnum.status3.getCode());
                        orderRefund.setRefundFailReason("退款失败！");
                        orderRefundService.editData(orderRefund);
                    }
                    // 退款通知
                    // 代理参数
                    BsAgentApiParam apiParam = agentApiParamService.getParamByAppId(request.getAppId());
                    if (apiParam != null && StringUtils.isNotBlank(apiParam.getOrderRefundNotify())) {
                        // 通知数据
                        OrderRefundNotifyModel refundNotify = new OrderRefundNotifyModel();
                        refundNotify.setOrderNo(orderRefund.getOrderNo());
                        refundNotify.setChannelType(gasOrder.getChannelType());
                        refundNotify.setChannelOrderNo(gasOrder.getChannelOrderNo());
                        refundNotify.setRefundResult(orderRefund.getRefundStatus() == 2 ? true : false);
                        refundNotify.setRefundFailReason(orderRefund.getRefundFailReason());
                        refundNotify.setSign(SignatureUtil.createSign(refundNotify, apiParam.getAppSecret()));
                        HttpsUtils.doPost(apiParam.getOrderRefundNotify(), JSONObject.toJSONString(refundNotify));
                    }
                }
            }
        });
        return "ok";
    }

    /**
     * 查询订单详情
     * @param request
     * @return
     * @throws Exception
     */
    public ResponseOrderDetailModel queryOrderDetail(RequestOrderDetailModel request) throws Exception {
        // 查询加油订单
        BsGasOrder gasOrder = gasOrderService.getDetailByOrderNo(request.getOrderNo());
        if (gasOrder == null) {
            throw ErrorHelp.genException(SysCode.OpenApi, ErrorCode.COMMON_ERROR, "未找到订单号");
        }
        ResponseOrderDetailModel response = new ResponseOrderDetailModel();
        response.setOrderNo(gasOrder.getOrderNo());
        response.setUserPhone(gasOrder.getUserPhone());
        response.setRefuelPrice(gasOrder.getGasRefuelPrice());
        response.setGasNo(gasOrder.getMerNo());
        response.setGasName(gasOrder.getMerName());
        response.setOilNo(gasOrder.getGasOilNo());
        response.setGunNo(gasOrder.getGasGunNo());
        response.setLitre(gasOrder.getGasOilLiters());
        response.setPriceOfficial(gasOrder.getGasPriceOfficial());
        response.setPriceGun(gasOrder.getGasPriceGun());
        response.setStatus(gasOrder.getStatus());
        response.setChannelType(gasOrder.getChannelType());
        response.setChannelOrderNo(gasOrder.getChannelOrderNo());
        response.setCreateTime(gasOrder.getCreateTime());
        return response;
    }

    /**
     * 油站业务
     * @param gasOrder
     */
    public BsGasOrder oilHandle(BsGasOrder gasOrder) throws Exception {
        // 查询加油订单
        if (gasOrder != null) {
            gasOrder.setActualPayPrice(gasOrder.getGasRefuelPrice());
            gasOrder.setPayType(OrderPayTypeEnum.type4.getCode());
            gasOrder.setPayTime(new Date());
            gasOrderService.updateGasOrder(gasOrder);

            if (gasOrder.getChannelType().equals(MerchantSourceTypeEnum.type1.getNumber())) {
                // 自建站
                // 打印小票
                if (gasOrder.getChannelType().equals(MerchantSourceTypeEnum.type1.getNumber())) {
                    deviceService.printGasOrder(gasOrder.getMerId(), gasOrder, false);
                }
                // 查询油站
                BsMerchant merchant = merchantService.getMerchant(gasOrder.getMerNo());
                if (merchant != null) {
                    // 连锁商户
                    if (merchant.getChainBrandId() != null) {
                        if (merchant.getChainBrandSettleType().equals(MerchantChainBrandSettleTypeEnum.type1.getNumber())) {
                            // 连锁额度
                            Map<String,Object> consumeMap = new HashMap<>();
                            consumeMap.put("sourceType", MerchantAccountRecordSourceTypeEnum.type2.getType());
                            consumeMap.put("sourceId", gasOrder.getId());
                            consumeMap.put("sourceOrderNo", gasOrder.getOrderNo());
                            merchantChainBrandAccountService.consume(merchant.getChainBrandId(), gasOrder.getMerNo(), gasOrder.getGasSettlePrice(), consumeMap);

                        } else if (merchant.getChainBrandSettleType().equals(MerchantChainBrandSettleTypeEnum.type2.getNumber())) {
                            // 商户额度
                            Map<String,Object> consumeMap = new HashMap<>();
                            consumeMap.put("sourceType", MerchantAccountRecordSourceTypeEnum.type2.getType());
                            consumeMap.put("sourceId", gasOrder.getId());
                            consumeMap.put("sourceOrderNo", gasOrder.getOrderNo());
                            merchantAccountService.consume(gasOrder.getMerNo(), gasOrder.getGasSettlePrice(), consumeMap);
                        }
                    } else {
                        // 商户额度
                        Map<String,Object> consumeMap = new HashMap<>();
                        consumeMap.put("sourceType", MerchantAccountRecordSourceTypeEnum.type2.getType());
                        consumeMap.put("sourceId", gasOrder.getId());
                        consumeMap.put("sourceOrderNo", gasOrder.getOrderNo());
                        merchantAccountService.consume(gasOrder.getMerNo(), gasOrder.getGasSettlePrice(), consumeMap);
                    }
                }

            } else if (gasOrder.getChannelType().equals(MerchantSourceTypeEnum.type2.getNumber())) {
                try {
                    // 能链【团油】
                    Map<String, Object> param = new HashMap<>();
                    param.put("gasId", gasOrder.getMerNo());
                    param.put("oilNo", gasOrder.getGasOilNo());
                    param.put("gunNo", Integer.parseInt(gasOrder.getGasGunNo()));
                    param.put("priceGun", gasOrder.getGasPriceGun());  // 油站价
                    param.put("priceVip", gasOrder.getGasPriceCost());  // 成本价
                    param.put("driverPhone", gasOrder.getUserPhone());
                    param.put("thirdSerialNo", gasOrder.getOrderNo());
                    param.put("refuelingAmount", gasOrder.getGasRefuelPrice());
                    // 查询账户
                    SecDictionary newLinkAccount = dictionaryService.getDictionary("NEW_LINK_ACCOUNT", "" + gasOrder.getGasOilType());
                    if (newLinkAccount != null) {
                        param.put("accountNo", newLinkAccount.getCodeName());
                    }
                    JSONObject pushResult = NewLinkRequestService.refuelingOrderPush(param);
                    if (pushResult != null && pushResult.getString("code").equals("200")) {
                        gasOrder.setChannelOrderNo(pushResult.getJSONObject("result").getString("orderNo"));
                        gasOrder.setAbnormal(false);
                        gasOrder.setAbnormalContent(null);
                    } else {
                        gasOrder.setAbnormal(true);
                        gasOrder.setAbnormalContent(pushResult.getString("message"));
                    }
                } catch (Exception e) {
                    gasOrder.setAbnormal(true);
                    gasOrder.setAbnormalContent("推送订单未知异常！请稍后重新推送");
                }
                gasOrderService.updateGasOrder(gasOrder);

            } else if (gasOrder.getChannelType().equals(MerchantSourceTypeEnum.type3.getNumber())) {
                try {
                    // 推送加好油【重庆壳牌】
                    CqShellPetroleumRequestService.gasSyncPayment(gasOrder.getOrderNo(),
                            gasOrder.getMerNo(),
                            gasOrder.getPayTime(),
                            gasOrder.getGasRefuelPrice(),
                            gasOrder.getGasOilNo(),
                            gasOrder.getGasGunNo(),
                            gasOrder.getPayablePrice(),
                            gasOrder.getTotalDeductionPrice()
                    );
                    gasOrder.setAbnormal(false);
                    gasOrder.setAbnormalContent(null);

                } catch (BaseException e) {
                    gasOrder.setAbnormal(true);
                    gasOrder.setAbnormalContent(e.getErrorMsg());
                }  catch (Exception e) {
                    gasOrder.setAbnormal(true);
                    gasOrder.setAbnormalContent("推送订单未知错误！请稍后重新推送");
                }
                gasOrderService.updateGasOrder(gasOrder);
            }
        }
        return gasOrder;
    }

}
